@@ -3,7 +3,7 @@ r"""Python interactive completion using GNU readline 2.1.
 
     licensed under the Python Softare License 
 
-    (c) 2003-2005 Holger Krekel, hpk at codespeak net 
+    (c) 2003-2010 Holger Krekel, hpk at codespeak net 
 
    how the interactive completer works
    -------------------------------------
@@ -23,12 +23,15 @@ r"""Python interactive completion using GNU readline 2.1.
    configuration options! 
 """
 
-__version__='0.96'
+__version__='0.98'
 __author__ ='holger krekel <hpk@trillke.net>'
 __url__='http://codespeak.net/rlcompleter2/'
 
-import os, sys, re, inspect
-import __builtin__, keyword
+import os, sys, re, inspect, keyword
+try:
+    import __builtin__
+except ImportError:
+    import builtins as __builtin__
 
 # debug turned off for releases
 if 0:
@@ -66,10 +69,11 @@ class Config:
     except ImportError:
         mainmodule = None
 
-    def viewfilter(self, (name, obj)):
+    def viewfilter(self, nameobj):
         """ return true if the given name, object combination
             should be included in the visible completion-list.
         """
+        name, obj = nameobj
         if name.startswith('_'):
             return 0
         return 1
@@ -339,7 +343,8 @@ def globalscope(module):
     """return (interactive) global scope for a module."""
     scope = vars(__builtin__)
     scope.update(vars(module))
-    scope.update(keyword.kwdict)
+    for kw in keyword.kwlist:
+        scope[kw] = 1
     return scope
 
 def docstring(obj):
@@ -491,18 +496,19 @@ class TryParser:
                 tree = self.compiler.pycodegen.parse(expr, 'eval') # ) # , 'eval')
                 debug("parsing succeeded: "+ repr(expr))
                 self._raise_codegenerator(tree)
-                raise AssertionError,'for %s unexpected AST: %s' %(repr(expr), tree)
+                raise AssertionError('for %s unexpected AST: %s' %(repr(expr), tree))
 
             except (SyntaxError, self.parser.ParserError):
                 m = self.wordmatch.match(text)
                 text = text[m and m.span(1)[1] or 1:]
-            except self.compiler.pycodegen.InteractiveCodeGenerator, ic:
+            except self.compiler.pycodegen.InteractiveCodeGenerator:
+                ic = sys.exc_info()[1]
                 return ic.getCode()
             except:
                 import traceback
                 traceback.print_exc()
-                print "internal failure on", expr
-                print "send mail to hpk at codespeak net" 
+                print ("internal failure on %s" % expr)
+                print ("send mail to hpk at codespeak net")
             #    m = self.wordmatch.match(text)
             #    text = text[m and m.span(1)[1] or 1:]
         return parseable
@@ -633,7 +639,9 @@ class Completer:
             try:
                 self.construct(text)
             except:
-                self.rl_matches=['!!', 'Internal Error']
+                import traceback
+                traceback.print_exc()
+                self.rl_matches=['!!', 'INTERNAL ERROR']
 
             if len(self.rl_matches)==1:
                 match = self.rl_matches[0]
@@ -673,7 +681,8 @@ class Completer:
                 self.method_tokenize(text)
                 self.method_eval(text)
                 raise Error('nothing found, empty namespace?')
-            except Finish, fin:
+            except Finish:
+                fin = sys.exc_info()[1]
                 self.completions = fin.completions
                 if fin.__class__ == Error:
                     self.text_last = None
@@ -703,7 +712,9 @@ class Completer:
                 self.text = text
                 self.config = config
 
-            def __call__(self, ttype, ttoken, (srow,scol), (erow,ecol),line):
+            def __call__(self, ttype, ttoken, srowscol, erowecol, line):
+                srow, scol = srowscol
+                erow, ecol = erowecol 
                 #debug("got token: %s" % repr(ttoken))
                 self.context = '%s' % (line[max(0,scol-1):scol+4])
                 if ttype == token.ERRORTOKEN:
@@ -730,7 +741,8 @@ class Completer:
             eater = TokenEater(text, self.config)
             tokenize.tokenize(['',text].pop, eater)
 
-        except tokenize.TokenError, ex:
+        except tokenize.TokenError:
+            ex = sys.exc_info()[1]
             debug("ex:%s" % ex)
             msg = ex.args[0]
             if msg[:3]=='EOF'and msg[-6:]=='string':
@@ -794,22 +806,28 @@ def setup_readline_history(histfn):
         try:
             readline.write_history_file(histfn)
         except IOError:
-            print "bad luck, couldn't save readline history to", histfn
+            print ("bad luck, couldn't save readline history to %s" % histfn)
        
     import atexit
     atexit.register(save)
 
-def setup(histfn=os.getenv('HOME')+'/.pythonhist', verbose=1):
+def setup(histfn=None, button='tab',verbose=1):
+    if histfn is None: 
+        base = os.getenv('HOME')
+        if not base: 
+            import tempfile
+            base = tempfile.gettempdir()
+        histfn = os.path.join(base, '.pythonhist')
     import readline 
     completer = Completer()
     readline.set_completer_delims('')
     readline.set_completer(completer.rl_complete)
-    readline.parse_and_bind('tab: complete')
+    readline.parse_and_bind(button+': complete')
     if histfn:
         setup_readline_history(histfn)
     if verbose:
-        print "Welcome to rlcompleter2 %(__version__)s" % globals()
-        print "for nice experiences hit <tab> multiple times"
+        print ("Welcome to rlcompleter2 %s" % __version__)
+        print ("for nice experiences hit <%s> multiple times"%(button))
 
 if __name__ == '__main__':
     setup()
